library(Biostrings)
library(parallel)

parse_data <- function(file) {
  reversed_sequences <- Biostrings::readQualityScaledDNAStringSet(file)
  sequences <- Biostrings::reverseComplement(reversed_sequences)
  vj_segments <- union(
    readRDS("data/v_segments.rds"),
    readRDS("data/j_segments_phe.rds")
  )
  return(list(sequences, vj_segments))
}

match_id_sequence <- function(names, sequences, id) {
  matches <- grep(names, pattern = id)
  row <- matches[1]
  return(as.character(sequences[row]))
}

get_vj_sequence <- function(sequences, names, vdj_segments) {
  metadata <- unlist(strsplit(sequences, split = " "))
  v_identifier <- metadata[2]
  j_identifier <- metadata[3]
  v_sequence <- match_id_sequence(names, vdj_segments, id = v_identifier)
  j_sequence <- match_id_sequence(names, vdj_segments, id = j_identifier)
  return(c(v_sequence, j_sequence))
}

fetch_vj_sequences <- function(vdj_segments, sequences) {
  vj_sequences <- mclapply(names(sequences),
    names(vdj_segments),
    vdj_segments,
    FUN = get_vj_sequence,
    mc.cores = detectCores()
  )
  return(c(vj_sequences[1], vj_sequences[2]))
}

align_sequence <- function(sequence, vdj_segment) {
  return(Biostrings::pairwiseAlignment(
    subject = sequence,
    pattern = vdj_segment,
    type = "global-local",
    gapOpening = 1
  ))
}

get_hvr_sequences <- function(sequences, vdj_segments) {
  vj_sequences <- fetch_vj_sequences(vdj_segments, sequences)
  v_alignment <- parallel::mcmapply(sequences,
    vj_sequences[1],
    FUN = align_sequence
  )
  j_alignment <- parallel::mcmapply(sequences,
    vj_sequences[2],
    FUN = align_sequence
  )
}

data <- parse_data(file = "data/curesim_sequence.fastq")
hvr_sequences <- get_hvr_sequences(
  sequences = data[[1]],
  vdj_segments = data[[2]]
)